/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Preferences Toolbar 3.
 *
 * The Initial Developer of the Original Code is
 * Manuel Reimer <manuel.reimer@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2002-2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Karsten Dsterloh <mnenhy@tprac.de>
 *  Manuel Reimer <manuel.reimer@gmx.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// |
// |  get a pointer to the one-and-only global PrefBar object this.goPrefBar
// |
function GetPrefBarPointer() {
    // Don't init PrefBar in turbo mode!
    if ("arguments" in window &&
        window.arguments[1] &&
        window.arguments[1].indexOf("turbo=yes") != -1)
        return;

    // The "hiddenDOMWindow" hack gets more and more buggy.
    // To have a more stable way of loading our global stuff, PrefBar ships
    // a Component to do this task. If the Component exists, then trigger
    // it, now.
    if (!("goPrefBar" in this) &&
        ("@prefbar.mozdev.org/goprefbar;1" in Components.classes)) {
        this.goPrefBar = Components.classes["@prefbar.mozdev.org/goprefbar;1"]
                                   .getService().wrappedJSObject;
        goPrefBar.dump("goPrefBar loaded using Component");
        return;
    }

    // Component didn't exist or failed --> Continue with hiddenDOMWindow
    if (!("goPrefBar" in this)) {
        // Get the hidden window and attach PrefBar there if necessary.
        // Otherwise we would lose the global Components etc. object when
        // closing the window that contains the PrefBar object - and that'd
        // be pretty *BAD*!
        var oAppShell = Components.classes['@mozilla.org/appshell/appShellService;1']
            .getService(Components.interfaces.nsIAppShellService);
        var oHiddenWindow = oAppShell.hiddenDOMWindow;
        if (oHiddenWindow) {
            if (!("goPrefBar" in oHiddenWindow)) {
                // need new PrefBar object
                var oLoader = Components.classes["@mozilla.org/moz/jssubscript-loader;1"].getService(Components.interfaces.mozIJSSubScriptLoader);
                oLoader.loadSubScript("chrome://prefbar/content/prefbar-loader2.js", oHiddenWindow);
                oLoader.loadSubScript("chrome://prefbar/content/prefbar.js", oHiddenWindow.goPrefBar);
                oHiddenWindow.goPrefBar.Init();
            }

            if ("goPrefBar" in oHiddenWindow) {
                // reuse global PrefBar object
                this.goPrefBar = oHiddenWindow.goPrefBar;
                goPrefBar.dump("Using hidden PrefBar object here " + this);
            }
            else {
                // still no PrefBar object :(
                const ksNoPrefBar = "Error! No hidden PrefBar object found here " + this + "\n";
                dump(ksNoPrefBar);
                try {
                    Components.classes["@mozilla.org/consoleservice;1"]
                        .getService(Components.interfaces.nsIConsoleService)
                        .logStringMessage(ksNoPrefBar);
                }
                catch(ignored){}
            }
        }
    }
    else {
        goPrefBar.dump("Reusing hidden PrefBar object here " + this);
    }
}

GetPrefBarPointer();
